import os 
import glob
import shutil
from argparse import ArgumentParser
from multiprocessing import Pool, cpu_count
from pathlib import Path
import numpy as np 
from PIL import Image

parser = ArgumentParser()
parser.add_argument("--base_folder")
parser.add_argument("--output_folder")
parser.add_argument("--limit", default=20000)
parser.add_argument("--num_shards", default=5)
parser.add_argument("--numpy_seed", default=10)
parser.add_argument("--size", default=128, dtype=int)

args = parser.parse_args()

print(f"Setting numpy seed to {args.numpy_seed}")
np.random.seed(int(args.numpy_seed))

print(f"Creating the output folder: {args.output_folder} ... ")
os.makedirs(args.output_folder, exist_ok=True)

input_img_list = sorted(glob.glob(os.path.join(args.base_folder, '**/*.[jpJP][pnPN]*[gG]'), recursive=True))
print(f"Found #{len(input_img_list)} images ... ")

num_shards = args.num_shards
if num_shards is None: 
    print(f"Setting number of shards to 1")
    num_shards = 1 

for shard_num in range(int(num_shards)):
    print(f"Working on shard number {shard_num} ... ")
    if args.limit is not None: 
        img_count_limit = int(args.limit)
        print(f"Randomly selecting {img_count_limit} images ... ")

        np.random.shuffle(input_img_list)
        
        input_img_list = input_img_list[:img_count_limit]


    base_folder = args.base_folder
    output_folder = args.output_folder 
    output_folder = output_folder + f"_shard{shard_num}"
    os.makedirs(output_folder, exist_ok=True)

    def copy2base(file_path):
        # Getting the filename
        target_name = file_path[len(base_folder)+1:]
        target_name = target_name.replace('/', "_")
        target_path = Path(output_folder) / target_name

        
        Image.open(file_path).resize((args.size, args.size), Image.Resampling.LANCZOS).save(target_path)

        #shutil.copy(file_path, target_path)

    pool = Pool(max(cpu_count()//2, 1))

    pool.map(
        copy2base, 
        input_img_list
    )